#include "points_without_bonus.hpp"

#include "dynexite_exam.hpp"

template <GradingType Grading>
void PointsWithoutBonus<Grading>::recalculate() {

    const auto &examRef = std::get<const DynexiteExam &>(this->mParentRefs);

    mEarnedPoints = QList<Points>(examRef.mIdentifier.size());
    mMaxPoints = Points{0.};

    // sum up mEarnedPoints for every student and mMaxPoints for every task pool
    for (const auto &pool : examRef.mTaskPools) {
        if (pool.mEliminationStatus == TaskEliminationType::withdrawTask) {
            // Don't add anything to mEarnedPoints[studentIdx] or mMaxPoints

        } else {
            assert(pool.mEliminationStatus == TaskEliminationType::creditFull ||
                   pool.mEliminationStatus ==
                       TaskEliminationType::notEliminated);

            // sum up mMaxPoints
            mMaxPoints += pool.mMaxPoints[Grading];

            // add earned points to each student
            for (const auto &task : pool.mTasks) {
                for (int studentIdx = 0;
                     studentIdx < examRef.mIdentifier.size(); ++studentIdx) {

                    if (task->mParticipatedStudents.testBit(studentIdx)) {
                        if (pool.mEliminationStatus ==
                            TaskEliminationType::notEliminated) {

                            // Add earned points from that task
                            mEarnedPoints[studentIdx] +=
                                *task->mEarnedPoints[Grading][studentIdx];
                        } else {
                            // Add all achievable points from that task
                            mEarnedPoints[studentIdx] +=
                                task->mMaxPoints[Grading];
                        }
                    }
                }
            }
        }
    }
}

template struct PointsWithoutBonus<GradingType::Closed>;
template struct PointsWithoutBonus<GradingType::Open>;
template struct PointsWithoutBonus<GradingType::Combined>;
