#pragma once

#include <compare>
#include <type_traits>

template <typename T>
concept arithmetic = std::is_arithmetic_v<T>;

class QString;

struct Points {
    double p = 0.;

    [[nodiscard]] auto operator<=>(const Points &) const noexcept = default;

    constexpr Points &operator+=(const Points &rhs) noexcept {
        p += rhs.p;
        return *this;
    }
    constexpr Points &operator-=(const Points &rhs) noexcept {
        p -= rhs.p;
        return *this;
    }
    template <arithmetic Nr>
    constexpr Points &operator/=(const Nr &rhs) noexcept {
        p /= rhs;
        return *this;
    }
    template <arithmetic Nr>
    constexpr Points &operator*=(const Nr &rhs) noexcept {
        p *= rhs;
        return *this;
    }
};

[[nodiscard]] constexpr Points operator+(Points a, const Points &b) noexcept {
    a += b;
    return a;
}
[[nodiscard]] constexpr Points operator-(Points a, const Points &b) noexcept {
    a -= b;
    return a;
}

template <arithmetic Nr>
[[nodiscard]] constexpr Points operator/(Points a, const Nr b) noexcept {
    a /= b;
    return a;
}

[[nodiscard]] constexpr auto operator/(const Points a,
                                       const Points b) noexcept {
    return a.p / b.p;
}

template <arithmetic Nr>
[[nodiscard]] constexpr Points operator*(Points a, const Nr b) noexcept {
    a *= b;
    return a;
}

/**
 * \brief Interprets an empty string as 0.
 */
[[nodiscard]] Points stringToPoints(QString s) noexcept(false);
